@@ -1,5 +1,5 @@
 /* --------------------------------------------------------------------------
- * Copyright 2003-2011 (inclusive) Nathan Angelacos 
+ * Copyright 2003-2021 (inclusive) Nathan Angelacos 
  *                   (nangel@users.sourceforge.net)
  * 
  *   This file is part of haserl.
@@ -28,8 +28,10 @@ typedef struct {
 	int	size;		/* size of script in bytes 	*/	
  	uid_t	uid;		/* user owner			*/
 	gid_t	gid;		/* group owner			*/
-	char 	*buf;	 /* pointer to malloc'ed buffer	*/
-	size_t	curpos;		/* current position in buffer	*/	
+	char 	*buf;		/* pointer to malloc'ed buffer	*/
+	size_t	curpos;		/* current position in buffer	*/
+	int	bang_script;	/* true if script starts with #!*/
+	int	tokens;		/* number of tokens in script	*/
 	void    *next;		/* next script in our chain     */
 	} script_t;
 
