@@ -1,5 +1,5 @@
 /* --------------------------------------------------------------------------
- * Copyright 2003-2015 (inclusive) Nathan Angelacos 
+ * Copyright 2003-2021 (inclusive) Nathan Angelacos 
  *                   (nangel@users.sourceforge.net)
  * 
  *   This file is part of haserl.
@@ -42,7 +42,7 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
-#include <sys/fcntl.h>
+#include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
 #include <grp.h>
@@ -463,6 +463,8 @@ ReadCGIQueryString (list_t * env)
 int
 ReadCGIPOSTValues (list_t * env )
 {
+  char prefix[64];
+
   size_t content_length = 0;
   size_t max_len;
   int	urldecoding = 0;
@@ -481,26 +483,31 @@ ReadCGIPOSTValues (list_t * env )
 
   content_type = getenv(CONTENT_TYPE);
 
-  if ( ( content_type != NULL ) &&
-     ( strncasecmp ( content_type , "multipart/form-data", 19)) == 0 )
-     	{
-	  /* This is a mime request, we need to go to the mime handler */
-	  i = rfc2388_handler (env);
-	  return (i);
-	}
-
-  /* at this point its either urlencoded or some other blob */
-  if ( (content_type == NULL ) ||
-     ( strncasecmp (getenv (CONTENT_TYPE), "application/x-www-form-urlencoded", 33) == 0 ) )
+  if ( content_type != NULL )
 	{
-  	urldecoding = 1;
-	matchstr = "&";
+	 if ( strncasecmp ( content_type , "multipart/form-data", 19) == 0 )
+     		{
+	  		/* This is a mime request, we need to go to the mime handler */
+	  		i = rfc2388_handler (env);
+	  		return (i);
+		}
+	 if ( strncasecmp (getenv (CONTENT_TYPE), "application/x-www-form-urlencoded", 33) == 0 ) 
+		{
+			/* url encoded data in the payload */
+  			urldecoding = 1;
+			matchstr = "&";
+		}
 	}
 
+  /* otherwise, assume just a binary octet stream */
+  	// These were set in the variable definition - just leave them alone
+	// matchstr = "";
+	// urldecoding = 0;
 
   /* Allow 2MB content, unless they have a global upload set */
   max_len = ((global.uploadkb == 0) ? 2048 : global.uploadkb) *1024;
 
+
   	 s_buffer_init (&sbuf, 32768);
 	 sbuf.fh = STDIN;
 
@@ -514,8 +521,15 @@ ReadCGIPOSTValues (list_t * env )
 		buffer_add( &token, "body=", 5 );
 		}
 
+	/* Set the prefix to the request method */  
+  	if (getenv ("REQUEST_METHOD") != NULL)
+    		{
+  			snprintf (prefix, 64, "%s_", getenv("REQUEST_METHOD"));
+		} else {
+  			snprintf (prefix, 64, "REQUEST_");
+		}
 
-	  do
+	do
 	    {
 	      /* x is true if this token ends with a matchstr or is at the end of stream */
 	      x = s_buffer_read (&sbuf, matchstr);
@@ -555,7 +569,7 @@ ReadCGIPOSTValues (list_t * env )
 		  unescape_url ((char *) data);
 		}
 		myputenv (env, (char *) data, global.var_prefix);
-		myputenv (env, (char *) data, global.post_prefix);
+		myputenv (env, (char *) data, prefix);
 		if (token.data)
 		    {
 		      buffer_reset (&token);
@@ -629,19 +643,29 @@ parseCommandLine (int argc, char *argv[])
 }
 
 int
-BecomeUser (uid_t uid, gid_t gid)
+BecomeUser (script_t *scriptbuf)
 {
   /* This silently fails if it doesn't work */
-  /* Following is from Timo Teras */
-  if (getuid () == 0)
-    setgroups (1, &gid);
 
-  setgid (gid);
-  setgid (getgid ());
+  /* if we are not root, trying to run a root script,
+   * verify it looks like a script (pass 1)
+   */
+  if ( scriptbuf->uid == 0 ) { 
+	if ( scriptbuf->bang_script > 0 ) {
+		/* Then we change permissions */
+		setgid (scriptbuf->gid);
+		setgid (getgid ());
 
-  setuid (uid);
-  setuid (getuid ());
+		setuid (scriptbuf->uid);
+		setuid (getuid ());
 
+		/* Following is from Timo Teras */
+		setgroups (1, &(scriptbuf->gid));
+	} else {
+		die_with_message ( NULL, NULL, "Exception.");
+	}
+
+  }
   return (0);
 }
 
@@ -666,7 +690,7 @@ assignGlobalStartupValues ()
   global.cookie_prefix = "COOKIE_";
   global.haserl_prefix = "HASERL_";
   global.nul_prefix = "";
-
+  global.exec_name = "";
 }
 
 
@@ -732,6 +756,9 @@ main (int argc, char *argv[])
   haserl_buffer_init (&script_text);
 #endif
 
+  // assign exec_name to be the name of the interpreter 
+  global.exec_name = argv[0];
+
   /* if more than argv[1] and argv[1] is not a file */
   switch (argc)
     {
@@ -793,7 +820,7 @@ main (int argc, char *argv[])
 
   scriptchain = load_script (filename, NULL);
 /* drop permissions */
-  BecomeUser (scriptchain->uid, scriptchain->gid);
+  BecomeUser (scriptchain);
 
   /* populate the function pointers based on the shell selected */
   if (strcmp (global.shell, "lua") && strcmp (global.shell, "luac"))
@@ -875,6 +902,15 @@ main (int argc, char *argv[])
   if (strcmp (global.shell, "luac"))
     {
       tokenchain = build_token_list (scriptchain, NULL);
+      /* This is a second pass to make sure haserl running as root 
+       * is actually running a haserl script.  If the first script
+       * doesn't have any haserl tokens, assume it is not valid
+       */
+      if (getuid () == 0) {
+	      if ( scriptchain->tokens < 2 ) {
+		      die_with_message ( NULL, NULL, "Exception.");
+		}
+	}
       preprocess_token_list (tokenchain);
     }
 #endif
