@@ -1,5 +1,5 @@
 /* --------------------------------------------------------------------------
- * Copyright 2003-2011 (inclusive) Nathan Angelacos 
+ * Copyright 2003-2021 (inclusive) Nathan Angelacos 
  *                   (nangel@users.sourceforge.net)
  * 
  *   This file is part of haserl.
@@ -21,6 +21,9 @@
 #ifndef _HASERL_H
 #define _HASERL_H	1
 
+// should already be included - but BecomeUser needs this
+#include "h_script.h"
+
 
 /* Just a silly construct to contain global variables    */
 typedef struct
@@ -40,6 +43,7 @@ typedef struct
   int acceptall;                /* true if we'll accept POST data on 
   					GETs and vice versa 		     */
   int silent;                   /* true if we never print errors             */
+  char *exec_name;		/* pointer to argv[0]			     */
 } haserl_t;
 
 extern haserl_t global;
@@ -61,7 +65,7 @@ int LineToStr(char *string, size_t max);
 int ReadMimeEncodedInput(list_t *env);
 void PrintParseError(char *error, int linenum);
 int parseCommandLine(int argc, char *argv[]);
-int BecomeUser(uid_t uid, gid_t gid);
+int BecomeUser(script_t *scriptbuf);
 void assignGlobalStartupValues(void);
 void unlink_uploadlist (void);
 int main(int argc, char *argv[]);
